!=========================================================================
! Copyright (C) VMware, Inc. 1986-2011.  All Rights Reserved.
!
! $Id: bomnew.gs,v 1.7 2008-01-09 19:02:29 stever Exp $
!
! Description - Create reserved objects from Smalltalk, that are 
!   newly created in version 4.0 .  These are objects that may be created
!   directly in the target repository, if in repository conversion.
! 
! Repository upgrade and slowfilein both use this file.
!
!=========================================================================


! initialize some class categories  for the benefit of Geode methods
run
Object category: 'Kernel'.
Number category: 'Numbers'.
Collection category: 'Collections'.
Behavior category: 'Classes'.
ClassHistory category: 'Classes'.
^ true
%

!   Make SymbolList take either a SymbolDictionary
!   or a ObsoleteSymbolDictionary
!     As of 5.0, this patch deleted


! #_incompletePathTraversal now a canonical symbol
! This object signifies that a path could not be traversed to the end.
expectvalue %String
run
Globals removeKey: #_incompletePathTraversal ifAbsent:[ ^ 'not found' ] .
^ 'removed'
%

! install the method required for kernel class creation of classes with
! reserved OOPs
!
category: 'Repository Conversion'
method: Object
_unsafeSetOop: aPositiveInteger

  "aPositiveInteger must be the integer value of a 5.0 OopType" 

  <primitive: 385>
  ^ self _primitiveFailed: #_unsafeSetOop:
%
commit


! 41164 - rename UserProfile inst var from 'spare1' to 'flags' if necessary.
expectvalue %String
run
(((UserProfile instVarAt: 4) at: 7) = #flags)
  ifTrue:[ 'flags inst var already present in UserProfile class']
 ifFalse:[  
    (UserProfile instVarAt: 4) at: 7 put: #flags .
    'flags inst var was added to UserProfile class'
]
%


! Bug 41664
category: 'Updating'
method: UserProfile
disableStaleAccountChecks

"Disables the stale account checks for the receiver only.  The user
 account does not commit on login, and the last login time for the 
 receiver is not recorded.

 This method requiers write access to the DataCurator segment and
 the #OtherPassword privilege.

 Returns the receiver."

System myUserProfile _validatePrivilegeName: #OtherPassword .
flags := 1 .
^ self
%

! bug 41664
category: 'Updating'
method: UserProfile
enableStaleAccountChecks

"Enables the stale account checks for the receiver only if the checks
 were previously disabled by the #disableStaleAccountChecks method, and
 if stale account aging is turned on in the repository.

 This method requiers write access to the DataCurator segment and
 the #OtherPassword privilege.

 Returns the receiver."

System myUserProfile _validatePrivilegeName: #OtherPassword .
flags := nil .
^ self
%

category: 'Testing'
method: UserProfile
hasDisabledStaleAccountChecks

"Answer true if stale account checks have been disabled for the receiver.
 Otherwise answer false."

^ flags == 1
%
commit
